@@ -9,11 +9,11 @@ use base 'Exporter';
 our @EXPORT_OK = qw/encode decode/;
 
 use version;
-our $VERSION = 'v1.6.7';
+our $VERSION = 'v1.8.0';
 
 use Carp;
 use Config;
-use Scalar::Util qw/blessed/;
+use Scalar::Util qw/blessed looks_like_number/;
 
 use Moo 2.002004; # safer generated code
 use boolean;
@@ -30,6 +30,8 @@ my $bools_re = qr/::(?:Boolean|_Bool|Bool)\z/;
 
 use namespace::clean -except => 'meta';
 
+my $max_int32 = 2147483647;
+
 # Dependency-free equivalent of what we need from Module::Runtime
 sub _try_load {
     my ( $mod, $ver ) = @_;
@@ -404,17 +406,12 @@ sub clone {
 
 sub create_oid { return BSON::OID->new }
 
-#pod =method inflate_extjson
+#pod =method inflate_extjson (DEPRECATED)
 #pod
-#pod     use JSON::MaybeXS;
-#pod     $data = decode_json( $json_string );
-#pod     $bson->inflate_extjson( $data );
+#pod This legacy method does not follow the L<MongoDB Extended JSON|https://github.com/mongodb/specifications/blob/master/source/extended-json.rst>
+#pod specification.
 #pod
-#pod Given a hash reference, this method walks the hash, replacing any
-#pod L<MongoDB extended JSON|https://docs.mongodb.org/manual/reference/mongodb-extended-json/>
-#pod items with BSON type-wrapper equivalents.  Additionally, any JSON
-#pod boolean objects (e.g. C<JSON::PP::Boolean>) will be replaced with
-#pod L<boolean.pm|boolean> true or false values.
+#pod Use L</extjson_to_perl> instead.
 #pod
 #pod =cut
 
@@ -437,6 +434,281 @@ sub inflate_extjson {
     return $hash;
 }
 
+#pod =method perl_to_extjson
+#pod
+#pod     use JSON::MaybeXS;
+#pod     my $ext = BSON->perl_to_extjson($data, \%options);
+#pod     my $json = encode_json($ext);
+#pod
+#pod Takes a perl data structure (i.e. hashref) and turns it into an
+#pod L<MongoDB Extended JSON|https://github.com/mongodb/specifications/blob/master/source/extended-json.rst>
+#pod structure. Note that the structure will still have to be serialized.
+#pod
+#pod Possible options are:
+#pod
+#pod =for :list
+#pod * C<relaxed> A boolean indicating if "relaxed extended JSON" should
+#pod be generated. If not set, the default value is taken from the
+#pod C<BSON_EXTJSON_RELAXED> environment variable.
+#pod
+#pod =cut
+
+my $use_win32_specials = ($^O eq 'MSWin32' && $] lt "5.022");
+
+my $is_inf = $use_win32_specials ? qr/^1.\#INF/i : qr/^inf/i;
+my $is_ninf = $use_win32_specials ? qr/^-1.\#INF/i : qr/^-inf/i;
+my $is_nan = $use_win32_specials ? qr/^-?1.\#(?:IND|QNAN)/i : qr/^-?nan/i;
+
+sub perl_to_extjson {
+    my ($class, $data, $options) = @_;
+
+    local $ENV{BSON_EXTJSON} = 1;
+    local $ENV{BSON_EXTJSON_RELAXED} = $ENV{BSON_EXTJSON_RELAXED};
+    $ENV{BSON_EXTJSON_RELAXED} = $options->{relaxed};
+
+    if (not defined $data) {
+        return undef; ## no critic
+    }
+
+    if (blessed($data) and $data->can('TO_JSON')) {
+        my $json_data = $data->TO_JSON;
+        return $json_data;
+    }
+
+    if (not ref $data) {
+
+        if (looks_like_number($data)) {
+            if ($ENV{BSON_EXTJSON_RELAXED}) {
+                return $data;
+            }
+
+            if ($data =~ m{\A-?[0-9_]+\z}) {
+                if ($data <= $max_int32) {
+                    return { '$numberInt' => "$data" };
+                }
+                else {
+                    return { '$numberLong' => "$data" };
+                }
+            }
+            else {
+                return { '$numberDouble' => 'Infinity' }
+                    if $data =~ $is_inf;
+                return { '$numberDouble' => '-Infinity' }
+                    if $data =~ $is_ninf;
+                return { '$numberDouble' => 'NaN' }
+                    if $data =~ $is_nan;
+                my $value = "$data";
+                $value = $value / 1.0;
+                return { '$numberDouble' => "$value" };
+            }
+        }
+
+        return $data;
+    }
+
+    if (boolean::isBoolean($data)) {
+        return $data;
+    }
+
+    if (ref $data eq 'HASH') {
+        for my $key (keys %$data) {
+            my $value = $data->{$key};
+            $data->{$key} = $class->perl_to_extjson($value, $options);
+        }
+        return $data;
+    }
+
+    if (ref $data eq 'ARRAY') {
+        for my $index (0 .. $#$data) {
+            my $value = $data->[$index];
+            $data->[$index] = $class->perl_to_extjson($value, $options);
+        }
+        return $data;
+    }
+
+    if (blessed($data) and $data->isa('JSON::PP::Boolean')) {
+        return $data;
+    }
+
+    die sprintf "Unsupported ref value (%s)", ref($data);
+}
+
+#pod =method extjson_to_perl
+#pod
+#pod     use JSON::MaybeXS;
+#pod     my $ext = decode_json($json);
+#pod     my $data = $bson->extjson_to_perl($ext);
+#pod
+#pod Takes an
+#pod L<MongoDB Extended JSON|https://github.com/mongodb/specifications/blob/master/source/extended-json.rst>
+#pod data structure and inflates it into a Perl data structure. Note that
+#pod you have to decode the JSON string manually beforehand.
+#pod
+#pod Canonically specified numerical values like C<{"$numberInt":"23"}> will
+#pod be inflated into their respective C<BSON::*> wrapper types. Plain numeric
+#pod values will be left as-is.
+#pod
+#pod =cut
+
+sub extjson_to_perl {
+    my ($class, $data) = @_;
+    # top level keys are never extended JSON elements, so we wrap the
+    # _extjson_to_perl inflater so it applies only to values, not the
+    # original data structure
+    for my $key (keys %$data) {
+        my $value = $data->{$key};
+        $data->{$key} = $class->_extjson_to_perl($value);
+    }
+    return $data;
+}
+
+sub _extjson_to_perl {
+    my ($class, $data) = @_;
+
+    if (ref $data eq 'HASH') {
+
+        if ( exists $data->{'$oid'} ) {
+            return BSON::OID->new( oid => pack( "H*", $data->{'$oid'} ) );
+        }
+
+        if ( exists $data->{'$numberInt'} ) {
+            return BSON::Int32->new( value => $data->{'$numberInt'} );
+        }
+
+        if ( exists $data->{'$numberLong'} ) {
+            if (HAS_INT64) {
+                return BSON::Int64->new( value => $data->{'$numberLong'} );
+            }
+            else {
+                return BSON::Int64->new( value => Math::BigInt->new($data->{'$numberLong'}) );
+            }
+        }
+
+        if ( exists $data->{'$binary'} ) {
+            require MIME::Base64;
+            if (exists $data->{'$type'}) {
+                return BSON::Bytes->new(
+                    data    => MIME::Base64::decode_base64($data->{'$binary'}),
+                    subtype => hex( $data->{'$type'} || 0 ),
+                );
+            }
+            else {
+                my $value = $data->{'$binary'};
+                return BSON::Bytes->new(
+                    data    => MIME::Base64::decode_base64($value->{base64}),
+                    subtype => hex( $value->{subType} || 0 ),
+                );
+            }
+        }
+
+        if ( exists $data->{'$date'} ) {
+            my $v = $data->{'$date'};
+            $v = ref($v) eq 'HASH' ? $class->_extjson_to_perl($v) : _iso8601_to_epochms($v);
+            return BSON::Time->new( value => $v );
+        }
+
+        if ( exists $data->{'$minKey'} ) {
+            return BSON::MinKey->new;
+        }
+
+        if ( exists $data->{'$maxKey'} ) {
+            return BSON::MaxKey->new;
+        }
+
+        if ( exists $data->{'$timestamp'} ) {
+            return BSON::Timestamp->new(
+                seconds   => $data->{'$timestamp'}{t},
+                increment => $data->{'$timestamp'}{i},
+            );
+        }
+
+        if ( exists $data->{'$regex'} and not ref $data->{'$regex'}) {
+            return BSON::Regex->new(
+                pattern => $data->{'$regex'},
+                ( exists $data->{'$options'} ? ( flags => $data->{'$options'} ) : () ),
+            );
+        }
+
+        if ( exists $data->{'$regularExpression'} ) {
+            my $value = $data->{'$regularExpression'};
+            return BSON::Regex->new(
+                pattern => $value->{pattern},
+                ( exists $value->{options} ? ( flags => $value->{options} ) : () ),
+            );
+        }
+
+        if ( exists $data->{'$code'} ) {
+            return BSON::Code->new(
+                code => $data->{'$code'},
+                ( exists $data->{'$scope'}
+                    ? ( scope => $class->_extjson_to_perl($data->{'$scope'}) )
+                    : ()
+                ),
+            );
+        }
+
+        if ( exists $data->{'$undefined'} ) {
+            return undef; ## no critic
+        }
+
+        if ( exists $data->{'$dbPointer'} ) {
+            my $data = $data->{'$dbPointer'};
+            my $id = $data->{'$id'};
+            $id = $class->_extjson_to_perl($id) if ref($id) eq 'HASH';
+            return BSON::DBPointer->new(
+                '$ref' => $data->{'$ref'},
+                '$id' => $id,
+            );
+        }
+
+        if ( exists $data->{'$ref'} ) {
+            my $id = delete $data->{'$id'};
+            $id = $class->_extjson_to_perl($id) if ref($id) eq 'HASH';
+            return BSON::DBRef->new(
+                '$ref' => delete $data->{'$ref'},
+                '$id' => $id,
+                '$db' => delete $data->{'$db'},
+                %$data, # extra
+            );
+        }
+
+        if ( exists $data->{'$numberDecimal'} ) {
+            return BSON::Decimal128->new( value => $data->{'$numberDecimal'} );
+        }
+
+        # Following extended JSON is non-standard
+
+        if ( exists $data->{'$numberDouble'} ) {
+            if ( $data->{'$numberDouble'} eq '-0' && $] lt '5.014' && ! HAS_LD ) {
+                $data->{'$numberDouble'} = '-0.0';
+            }
+            return BSON::Double->new( value => $data->{'$numberDouble'} );
+        }
+
+        if ( exists $data->{'$symbol'} ) {
+            return BSON::Symbol->new(value => $data->{'$symbol'});
+        }
+
+        for my $key (keys %$data) {
+            my $value = $data->{$key};
+            $data->{$key} = $class->_extjson_to_perl($value);
+        }
+        return $data;
+    }
+
+    if (ref $data eq 'ARRAY') {
+        for my $index (0 .. $#$data) {
+            my $value = $data->[$index];
+            $data->[$index] = ref($value)
+                ? $class->_extjson_to_perl($value)
+                : $value;
+        }
+        return $data;
+    }
+
+    return $data;
+}
+
 #--------------------------------------------------------------------------#
 # legacy functional interface
 #--------------------------------------------------------------------------#
@@ -660,7 +932,7 @@ BSON - BSON serialization and deserialization
 
 =head1 VERSION
 
-version v1.6.7
+version v1.8.0
 
 =head1 SYNOPSIS
 
@@ -948,17 +1220,50 @@ generation away from any specific Object ID class and makes it an interface
 on a BSON codec.  Alternative BSON codecs should define a similar class
 method that returns an Object ID of whatever type is appropriate.
 
-=head2 inflate_extjson
+=head2 inflate_extjson (DEPRECATED)
+
+This legacy method does not follow the L<MongoDB Extended JSON|https://github.com/mongodb/specifications/blob/master/source/extended-json.rst>
+specification.
+
+Use L</extjson_to_perl> instead.
+
+=head2 perl_to_extjson
 
     use JSON::MaybeXS;
-    $data = decode_json( $json_string );
-    $bson->inflate_extjson( $data );
+    my $ext = BSON->perl_to_extjson($data, \%options);
+    my $json = encode_json($ext);
 
-Given a hash reference, this method walks the hash, replacing any
-L<MongoDB extended JSON|https://docs.mongodb.org/manual/reference/mongodb-extended-json/>
-items with BSON type-wrapper equivalents.  Additionally, any JSON
-boolean objects (e.g. C<JSON::PP::Boolean>) will be replaced with
-L<boolean.pm|boolean> true or false values.
+Takes a perl data structure (i.e. hashref) and turns it into an
+L<MongoDB Extended JSON|https://github.com/mongodb/specifications/blob/master/source/extended-json.rst>
+structure. Note that the structure will still have to be serialized.
+
+Possible options are:
+
+=over 4
+
+=item *
+
+C<relaxed> A boolean indicating if "relaxed extended JSON" should
+
+be generated. If not set, the default value is taken from the
+C<BSON_EXTJSON_RELAXED> environment variable.
+
+=back
+
+=head2 extjson_to_perl
+
+    use JSON::MaybeXS;
+    my $ext = decode_json($json);
+    my $data = $bson->extjson_to_perl($ext);
+
+Takes an
+L<MongoDB Extended JSON|https://github.com/mongodb/specifications/blob/master/source/extended-json.rst>
+data structure and inflates it into a Perl data structure. Note that
+you have to decode the JSON string manually beforehand.
+
+Canonically specified numerical values like C<{"$numberInt":"23"}> will
+be inflated into their respective C<BSON::*> wrapper types. Plain numeric
+values will be left as-is.
 
 =head1 FUNCTIONS
 
@@ -1112,6 +1417,14 @@ Threads are never recommended in Perl, but this module is thread safe.
 
 PERL_BSON_BACKEND – if set at compile time, this will be treated as a module name.  The module will be loaded and used as the BSON backend implementation.  It must implement the same API as C<BSON::PP>.
 
+=item *
+
+BSON_EXTJSON - if set, serializing BSON type wrappers via C<TO_JSON> will produce Extended JSON v2 output.
+
+=item *
+
+BSON_EXTJSON_RELAXED - if producing Extended JSON output, if this is true, values will use the "Relaxed" form of Extended JSON, which sacrifices type round-tripping for improved human readability.
+
 =back
 
 =head1 SEMANTIC VERSIONING SCHEME
@@ -1178,7 +1491,7 @@ Stefan G. <minimalist@lavabit.com>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Eric Daniels Olivier Duclos Pat Gunn Petr Písař Yury Zavarin Oleg Kostyuk
+=for stopwords Eric Daniels Olivier Duclos Pat Gunn Petr Písař Robert Sedlacek Thomas Bloor Yury Zavarin Oleg Kostyuk
 
 =over 4
 
@@ -1200,6 +1513,14 @@ Petr Písař <ppisar@redhat.com>
 
 =item *
 
+Robert Sedlacek <rs@474.at>
+
+=item *
+
+Thomas Bloor <tbsliver@shadow.cat>
+
+=item *
+
 Yury Zavarin <yury.zavarin@gmail.com>
 
 =item *
